;--------------------------------------------------------------------
;subject	: control logic
;uc		: at89s52
;speed clock	: 11.0592 mhz
;date		: 15:41 23/01/05
;--------------------------------------------------------------------

$mod52
$nopaging
$nosymbols

;port assignment
port0		equ p0
autodel		bit p0.2
autosend	bit p0.3
autoscan	bit p0.4

port1		equ p1
lcd		equ p1
sdad		bit p1.0
rs		bit p1.4
enlcd		bit p1.5
backlit		bit p1.6
keyen		bit p1.7

key1		bit p1.0
key2		bit p1.1
key3		bit p1.2
key4		bit p1.3

port2		equ p2

port3		equ p3
rxpc		bit p3.0		;rx from pc
txpc		bit p3.1		;tx to pc
rxgsm		bit p3.3		;rx from gsm
txgsm		bit p3.4		;tx to gsm
strd		bit p3.5
sdae		bit p3.6
sclke		bit p3.7
sclkd		bit p3.7


fstat		equ 28h
rxavail		bit 0
flagecho2pc	bit 1
flagblink	bit 2
flaglistmode	bit 3
flagprint	bit 4
flagauto	bit 5
flagfind	bit 6
flagexe		bit 7

inout		equ 30h
inport		data inout
outport		data inout+1

node1		equ 00000001b
node2		equ 00000010b
node3		equ 00000100b
node4		equ 00001000b
node5		equ 00010000b
node6		equ 00100000b
node7		equ 01000000b
node8		equ 10000000b

basevar		equ 38h

var		equ basevar
var1		data var
var2		data var+1
var3		data var+2
var4		data var+3
mask1		data var
mask2		data var+1
shiftcounter	data var+2
charbuf		data var+3

lengthascii	data var+8
lengthpdu	data var+9
rxbuffer	data var+10
counter20	data var+11
counter60	data var+12
index		data var+13
status		data var+14
switch		data var+15
counterpoll	data counter60

tmp		equ basevar
tmp0		data tmp
tmp1		data tmp0+1
tmp2		data tmp0+2
tmp3		data tmp0+3
tmp4		data tmp0+4
tmp5		data tmp0+5
tmp6		data tmp0+6
tmp7		data tmp0+7

date		equ tmp2
hour		equ tmp3
minute		equ tmp4

bufferbase	equ 58h
buffbaseascii	equ 58h
dalength	equ 50h
buffbaseda	equ 51h
maxpdu		equ 140
maxsms		equ 160

;constants
cr		equ 13
lf		equ 10
ctrlz		equ 26
esc		equ 27
timeconstant	equ -46080
timepoll	equ 50			;20
codebase	equ 0000h
stackbase	equ 07h
startdatafile	equ 1000h
startpgmfile	equ 1020h


		org codebase
		jmp init
		org 0003h
		reti
		org 000bh
		reti
		org 0013h
		reti
		org 001bh
		jmp t1interrupt
		org 0023h
		reti


;--------------------------------------------------------------------
;macro definition
;--------------------------------------------------------------------

dispstrosd	macro text
		mov dptr,#text
		call sendstr2pc
		endm

dispstrlcd1	macro text
		mov dptr,#text
		call strline1
		endm

dispstrlcd2	macro text
		mov dptr,#text
		call strline2
		endm

sendatcmd	macro text
		mov dptr,#text
		call sendstr2gsm
		endm

pollmode	macro
		clr ex1
		endm

intmode		macro
		setb ex1
		endm

gsmuartrxpoll	macro rto		;timeout = rto x 0.065s
		mov r1,#rto
		call uartrxpoll
		endm

reset3digit	macro
		mov tmp0,#'0'
		mov tmp1,#'0'
		mov tmp2,#'0'
		endm

;rolling tmp buffer tmp0<1<tmp2,acc
roll3digit	macro
		mov tmp0,tmp1
		mov tmp1,tmp2
		mov tmp2,a
		endm

;carry set if lowvalue < a < highvalue
inrange		macro lowvalue,highvalue
		cjne a,#lowvalue,$+3
		jbc cy,$+6
		cjne a,#highvalue,$+3
		endm


;--------------------------------------------------------------------
;initialization
;--------------------------------------------------------------------

init:		mov sp,#stackbase
		mov port0,#0ffh
		mov port1,#0ffh
		mov port2,#0ffh
		mov port3,#0ffh
		clr strd
		clr sclkd
		mov a,port0
		mov inport,a
		mov outport,a
		call send2port

		anl psw,#11100111b	;reg select bank0
		mov fstat,#0
		mov counter20,#20
		mov counter60,#timepoll
		setb flagprint
		call initlcd

;initialize serial com (used timer 2)
		mov rcap2h,#0ffh	;19200 bps
		mov rcap2l,#0eeh
		mov t2con,#34h
		mov scon,#52h

;initialize 16 bit timer0
		orl tmod,#01h
		setb tr0

;initialize 16 bit timer1
		mov th1,#high(timeconstant)
		mov tl1,#low(timeconstant)
		orl tmod,#10h
		setb et1
		setb tr1

;initialize int1 for rx gsm
		setb ex1
		setb it1
		setb ea
		setb px1

;initialize flag/variable
		clr rxavail
		clr flaglistmode
		setb flagauto
;		clr flagauto
;		clr flagecho2pc
		setb flagecho2pc
		setb flagblink


;--------------------------------------------------------------------
;program entry point
;--------------------------------------------------------------------

start:		mov dptr,#txtgsmevb
		call strline1
		mov dptr,#txtgsm2005
		call strline2
		call delay3s

		call initdatafile
		call initpgmfile
		call initgsm
		call checkgsm
		call delay1s
		mov r3,#3
		call selsmsmemory
		jc $+4
		djnz r3,$-5
		call delay1s

		call gettime
		call delay2s

		call getgsmbatt
		call delay2s

		call loadexeall


;---------------
standby:	dispstrlcd1 txtstandby	;standby mode
		jb flagblink,$+9
		mov dptr,#txtblank
		jmp $+6
		dispstrlcd2 txtscanning
		mov c,port0.0
		mov backlit,c

key1press:	jb key1,key2press
		clr a
		mov outport,a
		call send2port
key2press:	jb key2,key3press
		call readtime
		jc $+11
		dispstrlcd2 txtcantget
		jmp $+6
		call printtime
		call delay05s
		jnb key2,$
key3press:	jb key3,key4press
		call readgsmbatt
		jc $+11
		dispstrlcd2 txtcantget
		jmp $+6
		call printbatt
		call delay05s
		jnb key3,$
key4press:	nop


scaninput:	call readinstate
		jnc endscaninp
		call writemsg
		jnc endscaninp
		jb autosend,nosend
		call sendmsg
		jc $+5
		call sendmsg		;try again
nosend:		jb autodel,$+6
		call delmsg
endscaninp:

		call delay250ms
		cpl flagblink
		djnz counterpoll,standby
		mov counterpoll,#timepoll
		call ioout		;send output
		jb autoscan,scannewmsg
		call loadexeall
		call ioout		;send output
scannewmsg:	call findnewmsg
		jc $+4
		jmp standby

		call exemsg		;if found execute it
		mov a,outport
		call send2port
		jb autodel,$+6
		call delmsg

		jmp standby


loadexeall:	mov index,#'1'
		call printcrlf
		mov a,#'*'
		call sendchar2pc
		mov a,index
		call sendchar2pc
		call exemsg
		mov a,outport
		call send2port
		call delay05s
		inc index
		mov a,index
		cjne a,#'6',loadexeall+3
		ret

txtgsmevb:	db 'Evaluation Board',0
txtgsm2005:	db '8051 GSM Module ',0
txtstandby:	db 'Standby mode    ',0
txtscanning:	db 'SCAN ALL TASK...',0
txtcantget:	db 'Can`t get value ',0


;--------------------------------------------------------------------
;timer1 interrupt service
;memory used:
;--------------------------------------------------------------------

;timer1 interrupt service
t1interrupt:	reti
		push psw
		push acc
		mov th1,#high(timeconstant)
		mov tl1,#low(timeconstant)
		djnz counter20,endt1int
		jb flagauto,$+9
		mov counter20,#10
		jmp $+6
		mov counter20,#5
		cpl flagblink
		djnz counter60,endt1int
		mov counter60,#timepoll
endt1int:	pop acc
		pop psw
		reti


;--------------------------------------------------------------------
;routine
;--------------------------------------------------------------------

send2port:	setb keyen
		mov port2,a		;update output
		mov r7,#8
xferloop:	rlc a
		mov sdad,c
		setb sclkd
		clr sclkd
		djnz r7,xferloop
		setb strd
		clr strd
		clr keyen
		ret

$include(g_pccom.asm)
$include(g_convis.asm)
$include(g_buff.asm)
$include(g_dlylcd.asm)
$include(g_gsm.asm)
$include(g_atcmd.asm)
$include(g_pdu.asm)
$include(g_exemsg.asm)
$include(g_instat.asm)
$include(g_e2prom.asm)


;--------------------------------------------------------------------
;ioout send output o1-o8 when current time = setting time
;scan setting time and compare current time
;--------------------------------------------------------------------

eof		equ 0

ioout:		dispstrlcd1 txtreadpgm
		call readtime
		jc $+11
		dispstrlcd2 txtcantget
		jmp $+6
		call printtime
		call delay1s
		mov dptr,#startpgmfile
io0:		call rdeeprom
		swap a
		cjne a,#eof,io1
		ret

io1:		cjne a,#99h,$+6
		jmp skipday		;if date=99, everyday
		cjne a,date,io2
skipday:	inc dptr
		call rdeeprom
		swap a
		cjne a,hour,io3		;check hour
		inc dptr
		call rdeeprom
		swap a
		cjne a,minute,io4	;check minute
		inc dptr		;current time=set time
		call rdeeprom
		mov outport,a
		call send2port
		dispstrlcd2 txttmsetout
		call delay1s
		ret
io2:		inc dptr
io3:		inc dptr
io4:		inc dptr
		inc dptr
		jmp io0

txtreadpgm:	db 'Read PGM File...',0
txttmsetout:	db 'Timer SET OUTPUT',0

end